@@ -22,10 +22,12 @@ import os
 import sys
 from functools import partial
 import unittest
+import warnings
 
 from astroid import YES, builder, nodes, scoped_nodes, \
-     InferenceError, NotFoundError, NoDefault
+     InferenceError, NotFoundError, NoDefault, ResolveError
 from astroid.bases import BUILTINS, Instance, BoundMethod, UnboundMethod
+from astroid import __pkginfo__
 from astroid import test_utils
 from astroid.tests import resources
 
@@ -59,7 +61,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(self.module.getattr('__doc__')[0].value, 'test module for astroid\n')
         self.assertEqual(len(self.module.getattr('__file__')), 1)
         self.assertIsInstance(self.module.getattr('__file__')[0], nodes.Const)
-        self.assertEqual(self.module.getattr('__file__')[0].value, 
+        self.assertEqual(self.module.getattr('__file__')[0].value,
                          os.path.abspath(resources.find('data/module.py')))
         self.assertEqual(len(self.module.getattr('__dict__')), 1)
         self.assertIsInstance(self.module.getattr('__dict__')[0], nodes.Dict)
@@ -180,20 +182,43 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
     def test_file_stream_in_memory(self):
         data = '''irrelevant_variable is irrelevant'''
         astroid = test_utils.build_module(data, 'in_memory')
-        self.assertEqual(astroid.file_stream.read().decode(), data)
+        with warnings.catch_warnings(record=True):
+            self.assertEqual(astroid.file_stream.read().decode(), data)
 
     def test_file_stream_physical(self):
         path = resources.find('data/all.py')
         astroid = builder.AstroidBuilder().file_build(path, 'all')
         with open(path, 'rb') as file_io:
-            self.assertEqual(astroid.file_stream.read(), file_io.read())
+            with warnings.catch_warnings(record=True):
+                self.assertEqual(astroid.file_stream.read(), file_io.read())
 
-    def test_file_stream_cached(self):
-        # Test that file_stream is cached. Previously,
-        # a new file was opened each time file_stream was requested.
-        data = '''stub'''
-        astroid = test_utils.build_module(data, 'in_memory')
-        self.assertIs(astroid.file_stream, astroid.file_stream)
+    def test_file_stream_api(self):
+        path = resources.find('data/all.py')
+        astroid = builder.AstroidBuilder().file_build(path, 'all')
+        if __pkginfo__.numversion >= (1, 6):
+            # file_stream is slated for removal in astroid 1.6.
+            with self.assertRaises(AttributeError):
+                astroid.file_stream
+        else:
+            # Until astroid 1.6, Module.file_stream will emit
+            # PendingDeprecationWarning in 1.4, DeprecationWarning
+            # in 1.5 and finally it will be removed in 1.6, leaving
+            # only Module.stream as the recommended way to retrieve
+            # its file stream.
+            with warnings.catch_warnings(record=True) as cm:
+                warnings.simplefilter("always")
+                self.assertIsNot(astroid.file_stream, astroid.file_stream)
+            self.assertGreater(len(cm), 1)
+            self.assertEqual(cm[0].category, PendingDeprecationWarning)
+
+    def test_stream_api(self):
+        path = resources.find('data/all.py')
+        astroid = builder.AstroidBuilder().file_build(path, 'all')
+        stream = astroid.stream()
+        self.assertTrue(hasattr(stream, 'close'))
+        with stream:
+            with open(path, 'rb') as file_io:
+                self.assertEqual(stream.read(), file_io.read())
 
 
 class FunctionNodeTest(ModuleLoader, unittest.TestCase):
@@ -261,7 +286,7 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
 
     def test_format_args(self):
         func = self.module2['make_class']
-        self.assertEqual(func.args.format_args(), 
+        self.assertEqual(func.args.format_args(),
                          'any, base=data.module.YO, *args, **kwargs')
         func = self.module['four_args']
         self.assertEqual(func.args.format_args(), 'a, b, c, d')
@@ -379,7 +404,7 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
 
     def test_func_instance_attr(self):
         """test instance attributes for functions"""
-        data= """
+        data = """
             def test():
                 print(test.bar)
 
@@ -435,12 +460,36 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                     return staticmethod(f)
                 return wrapper
 
+            def long_classmethod_decorator(platform=None, order=50):
+                def wrapper(f):
+                    def wrapper2(f):
+                        def wrapper3(f):
+                            f.cgm_module = True
+                            f.cgm_module_order = order
+                            f.cgm_module_platform = platform
+                            return classmethod(f)
+                        return wrapper3(f)
+                    return wrapper2(f)
+                return wrapper
+
             def classmethod_decorator(platform=None):
                 def wrapper(f):
                     f.platform = platform
                     return classmethod(f)
                 return wrapper
 
+            def classmethod_wrapper(fn):
+                def wrapper(cls, *args, **kwargs):
+                    result = fn(cls, *args, **kwargs)
+                    return result
+
+                return classmethod(wrapper)
+
+            def staticmethod_wrapper(fn):
+                def wrapper(*args, **kwargs):
+                    return fn(*args, **kwargs)
+                return staticmethod(wrapper)
+
             class SomeClass(object):
                 @static_decorator()
                 def static(node, cfg):
@@ -454,6 +503,15 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                 @classmethod_decorator
                 def not_so_classmethod(node):
                     pass
+                @classmethod_wrapper
+                def classmethod_wrapped(cls):
+                    pass
+                @staticmethod_wrapper
+                def staticmethod_wrapped():
+                    pass
+                @long_classmethod_decorator()
+                def long_classmethod(cls): 
+                    pass
         """)
         node = astroid.locals['SomeClass'][0]
         self.assertEqual(node.locals['static'][0].type,
@@ -464,6 +522,12 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                          'method')
         self.assertEqual(node.locals['not_so_classmethod'][0].type,
                          'method')
+        self.assertEqual(node.locals['classmethod_wrapped'][0].type,
+                         'classmethod')
+        self.assertEqual(node.locals['staticmethod_wrapped'][0].type,
+                         'staticmethod')
+        self.assertEqual(node.locals['long_classmethod'][0].type,
+                         'classmethod')
 
 
 class ClassNodeTest(ModuleLoader, unittest.TestCase):
@@ -970,6 +1034,10 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                 __slots__ = dedent.__name__
             class Eight(object):
                 __slots__ = ("parens")
+            class Ninth(object):
+                pass
+            class Ten(object):
+                __slots__ = dict({"a": "b", "c": "d"})
         """)
         first = astroid['First']
         first_slots = first.slots()
@@ -985,7 +1053,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(second_slots[0].value, "a")
 
         third_slots = astroid['Third'].slots()
-        self.assertEqual(third_slots, [])
+        self.assertIsNone(third_slots)
 
         fourth_slots = astroid['Fourth'].slots()
         self.assertEqual(len(fourth_slots), 2)
@@ -995,19 +1063,124 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(fourth_slots[1].value, "b")
 
         fifth_slots = astroid['Fifth'].slots()
-        self.assertEqual(fifth_slots, [])
+        self.assertIsNone(fifth_slots)
 
         sixth_slots = astroid['Sixth'].slots()
-        self.assertEqual(sixth_slots, [])
+        self.assertIsNone(sixth_slots)
 
         seventh_slots = astroid['Seventh'].slots()
-        self.assertEqual(len(seventh_slots), 0)
+        self.assertIsNone(seventh_slots)
 
         eight_slots = astroid['Eight'].slots()
         self.assertEqual(len(eight_slots), 1)
         self.assertIsInstance(eight_slots[0], nodes.Const)
         self.assertEqual(eight_slots[0].value, "parens")
 
+        self.assertIsNone(astroid['Ninth'].slots())
+
+        tenth_slots = astroid['Ten'].slots()
+        self.assertEqual(len(tenth_slots), 2)
+        self.assertEqual(
+            [slot.value for slot in tenth_slots],
+            ["a", "c"])
+
+    @test_utils.require_version(maxver='3.0')
+    def test_slots_py2(self):
+        module = test_utils.build_module("""
+        class UnicodeSlots(object):
+            __slots__ = (u"a", u"b", "c")
+        """)
+        slots = module['UnicodeSlots'].slots()
+        self.assertEqual(len(slots), 3)
+        self.assertEqual(slots[0].value, "a")
+        self.assertEqual(slots[1].value, "b")
+        self.assertEqual(slots[2].value, "c")
+
+    def assertEqualMro(self, klass, expected_mro):
+        self.assertEqual(
+            [member.name for member in klass.mro()],
+            expected_mro)
+
+    @test_utils.require_version(maxver='3.0')
+    def test_no_mro_for_old_style(self):
+        node = test_utils.extract_node("""
+        class Old: pass""")
+        with self.assertRaises(NotImplementedError) as cm:
+            node.mro()
+        self.assertEqual(str(cm.exception), "Could not obtain mro for "
+                                            "old-style classes.")
+
+    def test_mro(self):
+        astroid = test_utils.build_module("""
+        class C(object): pass
+        class D(dict, C): pass
+
+        class A1(object): pass
+        class B1(A1): pass
+        class C1(A1): pass
+        class D1(B1, C1): pass
+        class E1(C1, B1): pass
+        class F1(D1, E1): pass
+        class G1(E1, D1): pass
+
+        class Boat(object): pass
+        class DayBoat(Boat): pass
+        class WheelBoat(Boat): pass
+        class EngineLess(DayBoat): pass
+        class SmallMultihull(DayBoat): pass
+        class PedalWheelBoat(EngineLess, WheelBoat): pass
+        class SmallCatamaran(SmallMultihull): pass
+        class Pedalo(PedalWheelBoat, SmallCatamaran): pass
+
+        class OuterA(object):
+            class Inner(object):
+                pass
+        class OuterB(OuterA):
+            class Inner(OuterA.Inner):
+                pass
+        class OuterC(OuterA):
+            class Inner(OuterA.Inner):
+                pass
+        class OuterD(OuterC):
+            class Inner(OuterC.Inner, OuterB.Inner):
+                pass
+        
+        """)
+        self.assertEqualMro(astroid['D'], ['D', 'dict', 'C', 'object'])
+        self.assertEqualMro(astroid['D1'], ['D1', 'B1', 'C1', 'A1', 'object'])
+        self.assertEqualMro(astroid['E1'], ['E1', 'C1', 'B1', 'A1', 'object'])
+        with self.assertRaises(ResolveError) as cm:
+            astroid['F1'].mro()
+        self.assertEqual(str(cm.exception),
+                         "Cannot create a consistent method resolution order "
+                         "for bases (B1, C1, A1, object), "
+                         "(C1, B1, A1, object)")
+
+        with self.assertRaises(ResolveError) as cm:
+            astroid['G1'].mro()
+        self.assertEqual(str(cm.exception),
+                         "Cannot create a consistent method resolution order "
+                         "for bases (C1, B1, A1, object), "
+                         "(B1, C1, A1, object)")
+
+        self.assertEqualMro(
+            astroid['PedalWheelBoat'],
+            ["PedalWheelBoat", "EngineLess",
+             "DayBoat", "WheelBoat", "Boat", "object"])
+
+        self.assertEqualMro(
+            astroid["SmallCatamaran"],
+            ["SmallCatamaran", "SmallMultihull", "DayBoat", "Boat", "object"])
+
+        self.assertEqualMro(
+            astroid["Pedalo"],
+            ["Pedalo", "PedalWheelBoat", "EngineLess", "SmallCatamaran",
+             "SmallMultihull", "DayBoat", "WheelBoat", "Boat", "object"])
+
+        self.assertEqualMro(
+            astroid['OuterD']['Inner'],
+            ['Inner', 'Inner', 'Inner', 'Inner', 'object'])
+
 
 if __name__ == '__main__':
     unittest.main()
